@@ -14,14 +14,13 @@ from tgext.admin.controller import AdminController
 from repoze.what import predicates
 from webob.exc import HTTPNotFound
 
-from stroller.lib import stroller_url, style_css, send_email
-from stroller.lib.cart import results, get_cart
+from stroller.lib import stroller_url, style_css, send_email, confirm_css
+from stroller.lib.cart import results, get_cart, drop_cart
 from stroller.model.commerce import *
-from stroller.model.core import DBSession
+from stroller.model.core import DBSession, User, Group
 from stroller.controllers.payment import PaymentController
 from stroller.controllers.manage import ManageController
 from stroller.controllers.widgets import QuantityOtherSingleSelectField
-from stroller import model
 
 from tw.jquery import jquery_js
 
@@ -43,7 +42,7 @@ class StrollerController(TGController):
         front_categories = DBSession.query(Category).filter_by(featured=True).all()
         featured = DBSession.query(Product).filter_by(featured=True).all()
         favourites = DBSession.query(Product).order_by(Product.bought.desc())[:6]
-            
+
         style_css.inject()
         return results(front_categories=front_categories,
                         featured=featured, favourites=favourites,
@@ -78,7 +77,21 @@ class StrollerController(TGController):
             raise HTTPNotFound()
 
         return category.icon
-        
+
+    @require(predicates.not_anonymous())
+    @expose('stroller.templates.recap')
+    def recap(self, uid=None, force_user=None):
+        style_css.inject()
+
+        user = request.identity['user']
+        if force_user and bool(predicates.in_group('stroller')):
+            user = DBSession.query(User).filter_by(user_id=force_user).first()
+
+        orders = Order.find_by_data('user_id', user.user_id).order_by(Order.uid.desc()).all()
+        if not uid and orders:
+            uid = orders[0].uid
+        return dict(user=user, orders=orders, selected_order=uid)
+
     @expose('stroller.templates.product')
     def product(self, uid=None):
         product = DBSession.query(Product).filter_by(uid=uid).first()
@@ -130,36 +143,101 @@ class StrollerController(TGController):
             raise HTTPNotFound()
 
         return product.photo
-        
+
+    @expose('stroller.templates.cart')
+    def cart(self):
+        cart = get_cart()
+
+        if cart.confirmed:
+            flash(_('Order has been already confirmed and cannot be changed anymore'))
+            drop_cart()
+            return redirect(stroller_url('/'))
+
+        if not cart.total_quantity:
+            flash(_('Your cart is empty'))
+            return redirect(stroller_url('/'))
+
+        confirm_css.inject()
+        return dict(cart=cart)
+
     @expose('json')
     def add_to_cart(self, product=None, qnt=1):
+        try:
+            qnt = int(qnt)
+        except:
+            qnt = 0
+
         cart = get_cart()
-        product = Product.get(product)
-        if product and cart:
-            if (product.stock == None) or (product.stock):
-                cart.add_item(product, int(qnt))
-            else:
-                flash(_('Product is currently not available'))
+        if qnt > 0:
+            product = Product.get(product)
+            if product and cart:
+                if product.in_stock(1):
+                    cart.add_item(product, qnt)
+                else:
+                    flash(_('Product is currently not available'))
             
-        return results(cart=cart, price=cart.total_price, 
-                        quantity=cart.total_quantity)
-        
+        return results(price=cart.total_price, quantity=cart.total_quantity)
+
+    @expose('json')
+    def del_from_cart(self, product=None, qnt=1):
+        try:
+            qnt = int(qnt)
+        except:
+            qnt = 0
+
+        cart = get_cart()
+        if qnt > 0:
+            product = Product.get(product)
+            if product and cart:
+                cart.del_item(product, qnt)
+
+        return results(price=cart.total_price, quantity=cart.total_quantity)
 
     @expose()
     def conclude(self, order=None):
         order = Order.get(order)
-        if not order.dispatched and order.payed:
-            flash(_('Payment successfully registered.'))
-            order.dispatched = True
-        if not order.payed:
-            stroller_order_mail = config.get('stroller_order_notify_mail', None)
-            if not stroller_order_mail:
-                flash(_('Order registered but not payed, Please provide your order id either by telephone or fax to proceed with payment. Order ID: %s') % order.uid)
+        if not order:
+            flash(_('Order not found'))
+            return redirect(stroller_url('/'))
+
+        if not order.confirmed:
+            flash(_('Order has never been confirmed'))
+            return redirect(stroller_url('/'))
+
+        if order.dispatched:
+            flash(_('Order has already been dispatched'))
+            return redirect(stroller_url('/'))
+
+        stroller_order_mail = config.get('stroller_order_notify_mail', None)
+        stroller_email_sender = config.get('stroller_email_sender', stroller_order_mail)
+        customer = order.get_data_string('ship_name') or '<anonymous>'
+        order_email =  order.get_data_string('email')
+        customer_email = order_email or '<unknown email>'
+
+        if stroller_order_mail:
+            #Send email to manager to notify that user has bought an order
+            send_email(stroller_email_sender, stroller_order_mail, '{0} placed an order'.format(customer),
+                '''{0} placed order {1}'''.format(customer_email, stroller_url('/manage/show_order', qualified=True, uid=order.uid)))
+
+        #If we have users enabled send an email to user to notify about his order
+        if stroller_order_mail and order_email:
+            if order.payed:
+                end_message = 'Order registered as successfully payed.'
             else:
-                stroller_email_sender = config.get('stroller_email_sender', stroller_order_mail)
-                customer = request.identity and unicode(request.identity['user']) or 'anonymous'
-                customer_email = request.identity and request.identity['user'].email_address or 'anonymous'
-                send_email(stroller_email_sender, stroller_order_mail, '{0} placed an order'.format(customer), 
-                           '''{0} placed order {1}'''.format(customer_email, stroller_url('/manage/show_order', qualified=True, uid=order.uid))) 
-                flash(_('Order registered, you will be contacted back by the site manager to confirm it'))
+                end_message = 'Please proceed with the payment as described on the web site as soon as possible.'
+
+            if order.get_data('user_id'):
+                end_message += '\n\nCheck your order at {0}'.format(stroller_url('/recap', qualified=True, uid=order.uid))
+
+            send_email(stroller_email_sender, customer_email, 'Order successfully placed', '''
+Your order of {0} items, for a total of {1} {2} has been successfully registered.
+
+{3}
+'''.format(order.total_quantity, order.total_price, config.get('currency', 'EUR'), end_message))
+
+        if not order.payed and (not stroller_order_mail or not order_email):
+            flash(_('Order successfully registered, Please provide your order id either by telephone or fax to proceed with payment. Order ID: %s') % order.uid)
+        else:
+            flash(_('Order successfully registered'))
+
         return redirect(stroller_url('/'))
